{-----------------------------------------------------------------------------
 Unit Name: pbCocoSets
 Author:     Originally Pat Terry (SETS.PAS), ported to Delphi by Pat Connors,

         General set handling primitives
-----------------------------------------------------------------------------}
unit pbCocoSets;

interface

uses
  Windows, Messages, SysUtils, Classes, pbStrLib, pbCocoConstants;


type
  // Type BITSET holds values representing characters( only? - or does it hash strings also)
  // They are related by virtue of ???
  // Why would it not be the equivalent of a dynamic array - rather than using the hash mechanism?
  //  Is it because dynamic arrays were tricky bastards in TP7?
  //  - Or is it because of the time it takes to look them up?

  BITSET = set of 0..Set_size - 1;
  BITARRAY = array[0..Set_Limit] of BITSET;

procedure Clear(var s: BITARRAY); (* s := {}       *)
procedure Fill(var s: BITARRAY); (* s := full set *)
function IsIn(var s: BITARRAY; x: Integer): boolean; (* x IN s ?      *)
procedure Incl(var s: BITARRAY; x: Integer); (* INCL(s, x)    *)
procedure Excl(var s: BITARRAY; x: Integer); (* EXCL(s, x)    *)
function Includes(var s1, s2: BITARRAY): boolean; (* s2 <= s1 ?    *)
function Elements(var s: BITARRAY; (* | s |         *) var lastElem: Integer): Integer;
function Empty(var s: BITARRAY): boolean; (* s1 = {} ?     *)
function Equal(var s1, s2: BITARRAY): boolean; (* s1 = s2 ?     *)
function Different(var s1, s2: BITARRAY): boolean; (* s1 * s2 = 0 ? *)
procedure Unite(var s1, s2: BITARRAY); (* s1 := s1 + s2 *)
procedure Differ(var s1, s2: BITARRAY); (* s1 := s1 - s2 *)
procedure Intersect(var s1, s2, s3: BITARRAY); (* s3 := s1 * s2 *)
procedure Print(var f: TStringStream; s: BITARRAY; w, indent: Integer);

implementation

procedure Clear(var s: BITARRAY);
var
  i: integer;
begin
  i := 0;
  while i <= Set_Limit do
  begin
    s[i] := [];
    INC(i)
  end
end;

(* Fill                 Set all elements in set s
---------------------------------------------------------------------------*)

procedure Fill(var s: BITARRAY);
var
  i: integer;
begin
  i := 0;
  while i <= Set_Limit do
  begin
    s[i] := [0..Set_Size - 1];
    INC(i)
  end
end;

(* Incl                 Include element x in set s
---------------------------------------------------------------------------*)

procedure Incl(var s: BITARRAY; x: Integer);
begin
  s[x div Set_Size] := s[x div Set_Size] + [x mod Set_Size]
end;

(* Excl
---------------------------------------------------------------------------*)

procedure Excl(var s: BITARRAY; x: Integer);
begin
  s[x div Set_Size] := s[x div Set_Size] - [x mod Set_Size]
end;

(* IsIn                 TRUE, if element x is contained in set s
---------------------------------------------------------------------------*)

function IsIn(var s: BITARRAY; x: Integer): boolean;
begin
  IsIn := x mod Set_Size in s[x div Set_Size]
end;

(* Includes             TRUE, if s2 in s1
---------------------------------------------------------------------------*)

function Includes(var s1, s2: BITARRAY): boolean;
var
  i: integer;
begin
  i := 0;
  while i <= Set_Limit do
  begin
    if not (s2[i] <= s1[i]) then
    begin
      Includes := False;
      EXIT
    end;
    INC(i)
  end;
  Includes := True;
end;

(* Elements             Return number of elements in set s
---------------------------------------------------------------------------*)

function Elements(var s: BITARRAY; var lastElem: Integer): Integer;
var
  i, n, max: integer;
begin
  i := 0;
  n := 0;
  max := (Set_Limit + 1) * Set_Size;
  while i < max do
  begin
    if i mod Set_Size in s[i div Set_Size] then
    begin
      INC(n);
      lastElem := i
    end;
    INC(i)
  end;
  Elements := n
end;

(* Empty                TRUE, if set s i sempty
---------------------------------------------------------------------------*)

function Empty(var s: BITARRAY): boolean;
var
  i: integer;
begin
  i := 0;
  while i <= Set_Limit do
  begin
    if s[i] <> [] then
    begin
      Empty := False;
      EXIT
    end;
    INC(i)
  end;
  Empty := True
end;

(* Equal                TRUE, if set s1 and s2 are equal
---------------------------------------------------------------------------*)

function Equal(var s1, s2: BITARRAY): boolean;
var
  i: integer;
begin
  i := 0;
  while i <= Set_Limit do
  begin
    if s1[i] <> s2[i] then
    begin
      Equal := False;
      EXIT
    end;
    INC(i)
  end;
  Equal := True
end;

(* Different            TRUE, if set s1 and s2 are totally different
---------------------------------------------------------------------------*)

function Different(var s1, s2: BITARRAY): boolean;
var
  i: integer;
begin
  i := 0;
  while i <= Set_Limit do
  begin
    if s1[i] * s2[i] <> [] then
    begin
      Different := False;
      EXIT
    end;
    INC(i)
  end;
  Different := True
end;

(* Unite                s1 := s1 + s2
---------------------------------------------------------------------------*)

procedure Unite(var s1, s2: BITARRAY);
var
  i: integer;
begin
  i := 0;
  while i <= Set_Limit do
  begin
    s1[i] := s1[i] + s2[i];
    INC(i)
  end
end;

(* Differ               s1 := s1 - s2
---------------------------------------------------------------------------*)

procedure Differ(var s1, s2: BITARRAY);
var
  i: integer;
begin
  i := 0;
  while i <= Set_Limit do
  begin
    s1[i] := s1[i] - s2[i];
    INC(i)
  end
end;

(* Intersect            s3 := s1 * s2
---------------------------------------------------------------------------*)

procedure Intersect(var s1, s2, s3: BITARRAY);
var
  i: integer;
begin
  i := 0;
  while i <= Set_Limit do
  begin
    s3[i] := s1[i] * s2[i];
    INC(i)
  end
end;

(* Print                Print set s
---------------------------------------------------------------------------*)

procedure Print(var f: TStringStream; s: BITARRAY; w, indent: Integer);
var
  col, i, max, iSize: integer;
  S1: string;
begin
  i := 0;
  col := indent;
  max := (Set_Limit + 1) * Set_Size;
  iSize := 1;
  S1 := '{';
  f.Write(S1, iSize);
  while i < max do
  begin
    if IsIn(s, i) then
    begin
      if col + 4 > w then
      begin
        pbWriteLn(f, '');
        S1 := StringofChar(' ', indent);
        pbWrite(f, S1);
        col := indent
      end;
      S1 := IntToStr(i) + ',';
      pbWrite(f, S1);
      INC(col, 4)
    end;
    INC(i)
  end;
  S1 := '}';
  pbWrite(f, S1);
end;

end.
