{-----------------------------------------------------------------------------
 Unit Name: pbCocoFileSystem
 Author:    Pat Connors
 Purpose:
  File System for ParserBuilder - inherits from base class TpbFileSystem
-----------------------------------------------------------------------------}


unit pbCocoFileSystem;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, pbFileSystem, Dialogs, CompilerObj,
  pbStrLib, CocoImporter, DataFile;

type
  TpbCocoFileSystem = class(TpbFileSystem)
  private
    FFilter: string;
    FDefFileExt: string;
    FCompiler: TCompiler;
    fLogText: string;
    fPristineATG: string;
    FAutoSave: boolean;
    FAutoBackup: boolean;
    procedure SetDefFileExt(const Value: string);
    procedure SetFilter(const Value: string);
    procedure SetCompiler(const Value: TCompiler);
    function GetCopyOfName(const strFileName: string): string;
    procedure ImportCoco;
    function GetImportLog: string;
    procedure DefaultSaveAction;
    procedure SetAutoSave(const Value: boolean);
    procedure SetAutoBackup(const Value: boolean);
    { Private declarations }
  protected
    { Protected declarations }
    procedure SaveFile(const strFileName: string);
    procedure OpenFile;
    function SaveDialog: boolean;
    function OpenDialog: boolean;
    procedure DeleteOldHistory(DF: TDataFile);

    function Do_New: boolean; override;
    function Do_Open: boolean; override;
    function Do_Close: boolean; override;
    function Do_RenameOrReposition: boolean; override;
    function Do_MakeSnapShotCopy: boolean; override;
    function Do_MakeMilestone: boolean; override;
    function Do_RevertToMilestone: boolean; override;
    function Do_DocumentProperties: boolean; override;
    function Do_AbandonChanges: boolean; override;
    function CheckFileNameOK: boolean; override;
    function GetFileName: string; override;
    procedure SetFileName(const Value: string); override;
    procedure SetCurrentDelphiProject;
    procedure LoadProjectNameAndLangSample;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadFile;
    procedure LoadATGString(const ATGStr: string);
    procedure Save;
    procedure DoBackup;
    function IsSameText(ATGNew, ATGOld: string): boolean;
    procedure GetBackupTitles(SL: TStrings);
    function RetrieveBackup(const Title: string): string;
    function RetrieveOriginalBackup(const Title: string): string;
    procedure RollBack(const Version: string);
    procedure ClearHistory;
  published
    { Published declarations }
    property DefFileExt: string read FDefFileExt write SetDefFileExt;
    property Filter: string read FFilter write SetFilter;
    property Compiler: TCompiler read FCompiler write SetCompiler;
    property ImportLog: string read GetImportLog;
    property AutoSave: boolean read FAutoSave write SetAutoSave;
    property AutoBackup: boolean read FAutoBackup write SetAutoBackup;
  end;

implementation

uses WSStripperComponent;


const
  MaxBackups = 100;

  { TpbCocoFileSystem }

function TpbCocoFileSystem.CheckFileNameOK: boolean;
begin
  Result := FileName <> '';
end;

procedure TpbCocoFileSystem.ClearHistory;
var
  DF: TDataFile;
begin
  if FileExists(FCompiler.FileName) then
  begin
    DF := TDataFile.Create(ChangeFileExt(FCompiler.FileName, '.history'));
    DF.DeleteSection('ATG History');
    DF.Free;
  end;
end;

constructor TpbCocoFileSystem.Create(AOwner: TComponent);
begin
  inherited;
  FFilter := 'ATG Files (*.atg)|*.atg|Any Files (*.*)|*.*';
  LoadFileHistory;
  FAutoSave := True;
end;

procedure TpbCocoFileSystem.DefaultSaveAction;
begin
  if (FAutoSave) then
    if (FileExists(Filename)) then
      SaveFile(Filename);
  DoBackup;
end;

procedure TpbCocoFileSystem.DeleteOldHistory(DF: TDataFile);
var 
  idx: integer;
  SL: TStringList;
begin
  SL := TStringList.Create;
  DF.GetValueNames('ATG History', SL);
  if (SL.Count > MaxBackups) then
    for idx := SL.Count - (MaxBackups + 1) downto 0 do
      DF.Delete('ATG History', SL[idx]);
  SL.Free;
end;

destructor TpbCocoFileSystem.Destroy;
begin
  SaveFileHistory;
  inherited;
end;

procedure TpbCocoFileSystem.DoBackup;
var
  DF: TDataFile;
  BackupCount: integer;
  ATG, OLDATG, id: string;
  SL: TStringList;
begin
  if FileExists(FCompiler.FileName) then
  begin
    DF := TDataFile.Create(ChangeFileExt(FCompiler.FileName, '.history'));
    DF.WriteString('ProjectFileName', 'FileName', FCompiler.CurrentDelphiProject);
    DF.WriteString('LanguageSample', 'Sample', FCompiler.LanguageSample);
    DeleteOldHistory(DF);
    SL := TStringList.Create;

    try
      DF.GetValueNames('ATG History', SL);
      BackupCount := SL.Count;
      if (BackupCount > 0) then
      begin
        id := SL[SL.Count - 1];
        OLDATG := DF.ReadString('ATG History', id, '')
      end
      else
        OLDATG := ' ';
    finally
      SL.Free;
    end;
    ATG := fCompiler.GetATG;
    if not IsSameText(ATG, OLDATG) then
      DF.WriteString('ATG History', FormatDateTime('yyyy-mm-dd hh-mm', Now), ATG);
    DF.Free;
  end;
end;

function TpbCocoFileSystem.Do_AbandonChanges: boolean;
var
  CI: TCocoImporter;
begin
  DoBackup;
  fLogText := 'Importing Original ATG for ' + Filename + ' at ' + DateTimeToStr(NOW);
  CI := TCocoImporter.Create;
  try
    CI.Compiler := FCompiler;
    CI.ParseText(FCompiler.OriginalATG);
    fLogText := CI.Log;
    FCompiler.FileName := FileName;
    Result := True;
  finally
    CI.Free;
  end;
end;

function TpbCocoFileSystem.Do_Close: boolean;
begin
  Result := True;
  DoBackup;
  SaveFile(Filename);
  FileName := '';
  FCompiler.ClearAll;
end;

function TpbCocoFileSystem.Do_DocumentProperties: boolean;
begin
  Result := False;
  { TODO : Decide how document properties will be handled }
  MessageDlg('Yet to be implemented.', mtInformation, [mbOK], 0);
end;

function TpbCocoFileSystem.Do_MakeMilestone: boolean;
var
  S: string;
begin
  Result := False;
  if (CheckFileNameOK) then
  begin
    S := FileName + ' Milestone ' + FormatDateTime('yyyy-mm-dd hh-mm', Now);
    SaveFile(S);
    Result := True;
  end;
end;

function TpbCocoFileSystem.Do_MakeSnapShotCopy: boolean;
var
  S: string;
begin
  Result := False;
  if (CheckFileNameOK) then
  begin
    S := GetCopyOfName(FileName);
    SaveFile(S);
    Result := True;
  end;
end;

function TpbCocoFileSystem.Do_New: boolean;
  //Var S : String;
begin
  Result := False;
  DefaultSaveAction;
  UseFilename := False;
  if (SaveDialog) then
  begin
    FCompiler.ClearAll;
    SaveFile(Filename);
    FCompiler.FileName := FileName;
    Result := True;
  end;
end;

function TpbCocoFileSystem.Do_Open: boolean;
begin
  Result := False;
  DefaultSaveAction;
  UseFilename := False;
  if (OpenDialog) then
  begin
    LoadFile;
    Result := True;
  end;
end;

function TpbCocoFileSystem.Do_RenameOrReposition: boolean;
var 
  S, S1: string;
begin
  Result := False;
  S := ChangeFileExt(FCompiler.FileName, '.history');
  DefaultSaveAction;
  UseFilename := True;
  if (SaveDialog) then
  begin
    if not DirectoryExists(ExtractFilePath(FileName)) then
      ForceDirectories(ExtractFilePath(FileName));
    S1 := ChangeFileExt(Filename, '.history');
    if FileExists(S) then
      CopyFile(PChar(S), PChar(S1), False);
    SaveFile(Filename);
    Result := True;
  end;
end;

{-----------------------------------------------------------------------------
Function Procedure: TpbCocoFileSystem.Do_RevertToMileston : Booleane
  Author:    Administrator
  Date:      02-Apr-2002
  Arguments: None
  Result:    None

  1. Get the name of the ATG File
  2. Open up an Open Dialog with that as the filename mask,
       i.e.
-----------------------------------------------------------------------------}
function TpbCocoFileSystem.Do_RevertToMilestone: boolean;
var
  CurrentName: string;
begin
  Result := False;
  UseFilename := True;
  CurrentName := FCompiler.FileName;
  FileName := CurrentName + '*milestone*';
  if (OpenDialog) then
  begin
    ImportCoco;
    FCompiler.FileName := CurrentName;
    Result := True;
  end;
  FileName := CurrentName;
end;

procedure TpbCocoFileSystem.GetBackupTitles(SL: TStrings);
var
  DF: TDataFile;
begin
  DF := TDataFile.Create(ChangeFileExt(FCompiler.FileName, '.history'));
  DF.GetValueNames('ATG History', SL);
  DF.Free;
end;

function TpbCocoFileSystem.GetCopyOfName(const strFileName: string): string;
var
  S: string;
  idx: integer;
begin
  S := ExtractFilePath(strFileName) + 'Copy of ' + ExtractFileName(strFileName);
  idx := 2;
  while (FileExists(S)) and (idx < 11) do
  begin
    S := ExtractFilePath(strFileName) + GetIndexTitle(idx) + ' Copy of ' +
      ExtractFileName(strFileName);
    inc(idx)
  end;
  Result := S;
end;

function TpbCocoFileSystem.GetFileName: string;
begin
  if (FCompiler <> nil) then
  begin
    Result := FCompiler.FileName;
    FileName := Result;
  end
  else
    Result := '';
end;

function TpbCocoFileSystem.GetImportLog: string;
begin
  Result := fLogText;
end;

procedure TpbCocoFileSystem.ImportCoco;
var
  CI: TCocoImporter;
begin
  LoadString(fPristineATG, FileName);
  fLogText := 'Importing ' + Filename + ' at ' + DateTimeToStr(NOW);
  CI := TCocoImporter.Create;
  try
    CI.Compiler := FCompiler;
    CI.ParseText(fPristineATG);
    fLogText := CI.Log;
    FCompiler.FileName := FileName;
    SetCurrentDelphiProject;
    LoadProjectNameAndLangSample;
  finally
    CI.Free;
  end;
end;

function TpbCocoFileSystem.IsSameText(ATGNew, ATGOld: string): boolean;
var 
  S, S1: string;
begin
  ExecuteWSStripperParser(ATGNew, S);
  ExecuteWSStripperParser(ATGOld, S1);
  Result := SameText(S, S1);
  // Temporarily pulled out as it is too slow.
  //  if not Result then
  //  Begin
  //    ATGNew := ReplaceAll(ATGNew,#13#10,'');
  //    ATGNew := Trim(ReplaceAll(ATGNew,' ',''));
  //    ATGOld := ReplaceAll(ATGOld,#13#10,'');
  //    ATGOld := Trim(ReplaceAll(ATGOld,' ',''));
  //    Result :=  SameText(ATGNew,ATGOld);
  //  End;
end;

procedure TpbCocoFileSystem.LoadATGString(const ATGStr: string);
var
  CI: TCocoImporter;
begin
  fLogText := 'Importing ATG Text at ' + DateTimeToStr(NOW);
  CI := TCocoImporter.Create;
  try
    CI.Compiler := FCompiler;
    CI.ParseText(ATGStr);
    fLogText := CI.Log;
  finally
    CI.Free;
  end;
end;

procedure TpbCocoFileSystem.LoadFile;
begin
  Assert(FileName <> '', 'Blank File Name');
  ImportCoco;
end;

procedure TpbCocoFileSystem.LoadProjectNameAndLangSample;
var
  DF: TDataFile;
begin
  if (FileExists(ChangeFileExt(FCompiler.FileName, '.history'))) then
  begin
    DF := TDataFile.Create(ChangeFileExt(FCompiler.FileName, '.history'));
    FCompiler.CurrentDelphiProject := DF.ReadString('ProjectFileName', 'FileName', ' ');
    FCompiler.LanguageSample := DF.ReadString('LanguageSample', 'Sample', '');
    DF.Free;
  end;
end;

function TpbCocoFileSystem.OpenDialog: boolean;
var
  OD: TOpenDialog;
begin
  OD := TOpenDialog.Create(Self);
  try
    OD.Filter := Filter;
    OD.DefaultExt := DefFileExt;
    if (UseFilename) then
      OD.FileName := Filename
    else
      OD.FileName := '';
    OD.Title := OpenDlgCaption;
    Result := OD.Execute;
    if (Result) then
    begin
      Filename := OD.FileName;
      AddFilenameToHistory;
    end;
  finally
    OD.Free;
  end;
end;

procedure TpbCocoFileSystem.OpenFile;
begin
  UseFilename := False;
  if (OpenDialog) then
  begin
    ImportCoco;
  end;
end;

function TpbCocoFileSystem.RetrieveBackup(const Title: string): string;
var
  DF: TDataFile;
begin
  DF := TDataFile.Create(ChangeFileExt(FCompiler.FileName, '.history'));
  Result := DF.ReadString('ATG History', Title, '');
  DF.Free;
end;

function TpbCocoFileSystem.RetrieveOriginalBackup(const Title: string): string;
var
  DF: TDataFile;
begin
  DF := TDataFile.Create(ChangeFileExt(FCompiler.FileName, '.history'));
  Result := DF.ReadString('ATG History', Title, '');
  DF.Free;
end;

procedure TpbCocoFileSystem.RollBack(const Version: string);
var 
  S: string;
begin
  S := RetrieveOriginalBackup(Version);
  Save;
  SaveString(S, FCompiler.FileName);
  LoadFile;
end;

procedure TpbCocoFileSystem.Save;
begin
  SaveFile(FCompiler.FileName);
end;

function TpbCocoFileSystem.SaveDialog: boolean;
var
  SD: TSaveDialog;
begin
  SD := TSaveDialog.Create(Self);
  try
    SD.Filter := Filter;
    SD.DefaultExt := DefFileExt;
    if (UseFilename) then
      SD.FileName := Filename
    else
      SD.FileName := '';
    SD.Title := SaveDlgCaption;
    Result := SD.Execute and (CheckFileNameOK);
    FileName := SD.FileName;
    AddFilenameToHistory;
  finally
    SD.Free;
  end;
end;


procedure TpbCocoFileSystem.SaveFile(const strFileName: string);
var 
  S: string;
begin
  S := strFilename;
  Assert(fCompiler <> nil, 'Bug: Compiler property has not been set.');
  if (S = '') then
    S := ExtractFilePath(Application.ExeName) + 'Output\Untitled.atg';
  fCompiler.ResetAllEditFlags;
  DoBackup;
  SaveString(fCompiler.GetATG, S);
  AddFilenameToHistory;
end;

procedure TpbCocoFileSystem.SetAutoBackup(const Value: boolean);
begin
  FAutoBackup := Value;
end;

procedure TpbCocoFileSystem.SetAutoSave(const Value: boolean);
begin
  FAutoSave := Value;
end;

procedure TpbCocoFileSystem.SetCompiler(const Value: TCompiler);
begin
  FCompiler := Value;
end;

procedure TpbCocoFileSystem.SetCurrentDelphiProject;
var
  DF: TDataFile;
begin
  if (FileExists(ChangeFileExt(FCompiler.FileName, '.history'))) then
  begin
    DF := TDataFile.Create(ChangeFileExt(FCompiler.FileName, '.history'));
    FCompiler.CurrentDelphiProject := DF.ReadString('ProjectFileName', 'FileName', ' ');
    DF.Free;
  end;
end;

procedure TpbCocoFileSystem.SetDefFileExt(const Value: string);
begin
  FDefFileExt := Value;
end;

procedure TpbCocoFileSystem.SetFileName(const Value: string);
begin
  if (FCompiler <> nil) then
    fCompiler.FileName := Value;
end;

procedure TpbCocoFileSystem.SetFilter(const Value: string);
begin
  FFilter := Value;
end;

end.
